<!doctype html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link href="styles.css" rel="stylesheet">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css" rel="stylesheet"
          integrity="sha384-1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3" crossorigin="anonymous">
    <title>{{ results.title }}</title>
</head>
<body id="results">
{% include 'navbar.html' %}
<div class="album bg-light">
    <div class="container-fluid">
        <div id="noResultsAlert"></div>
        <div class="row list" id="resultslist" style="display:none">
            {% filter indent(width=12) -%}
            {% include 'result.html' %}
            {%- endfilter %}
        </div>
    </div>
</div>
<div class="offcanvas offcanvas-start" tabindex="-1"></div>
{% include 'filter.html' %}
<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/js/bootstrap.bundle.min.js"
        integrity="sha384-ka7Sk0Gln4gmtz2MlQnikT1wXgYsOg+OMhuP+IlRH9sENBO0LRn5q+8nbTov4+1p"
        crossorigin="anonymous"></script>
<script src="https://cdn.jsdelivr.net/npm/list.js@1.5.0/dist/list.min.js"
        integrity="sha256-YqOIp4MygNx9/lcWr5lpcR88Ki/MNMWvJJkH0eK+fHM="
        crossorigin="anonymous"></script>
<script src="extra.js"></script>
</body>
</html>
